/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.adm;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NameClassPair;

import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.RemoteObject;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.container.Container;
import org.objectweb.jonas.naming.NamingManager;
import org.objectweb.jonas.dbm.DataBaseManager;
import org.objectweb.jonas.container.EJBContainerManager;


/**
 * This class implements a remote interface used for administering
 *      the server.
 *
 */
public class Adm extends RemoteObject implements AdmInterface {

    static final int trace = Trace.DB_18;

    public static final String ADMNAME_SUFFIX = "_Adm";
    /**
     * @serial
     */
    EJBContainerManager ecm  = null;
    InitialContext      ictx = null;
    DataBaseManager     dbm  = null;
    /**
     * Adm constructor
     */
    public Adm(String jonasname) throws RemoteException  {
	Trace.outln(trace, "Adm.Adm() ");

	// keep the container
	ecm = EJBContainerManager.getInstance();
	dbm = DataBaseManager.getInstance();
	ictx = NamingManager.getInitialContext();

	// Registers the JOnAS Server Name to JNDI
	String name = jonasname + ADMNAME_SUFFIX;
	try {
	    NamingManager.rebind(name, this);
	    Trace.outln(trace, "rebind done " + name);
	} catch (Exception e) {
	    Trace.errln("Cannot register "+name+" : " + e);
	    System.exit(2);
	}
    }

    // ------------------------------------------------------------------
    //  AdmInterface implementation
    // ------------------------------------------------------------------

    /**
     * configure trace messages in EJB Server
     */
    public void configureTrace(String out, String err, String level) throws RemoteException {
	Trace.out(trace, "Adm.configureTrace(String out, String err, String level) ");
	Trace.configure(out, err, level);	
    }

    /**
     * Add beans
     */
    public void addBeans(Vector descNames) throws RemoteException {
	Trace.out(trace, "Adm.addBeans(Vector descNames) ");
	try {
	    ecm.createEJBContainers(descNames);
	}catch(Exception ex) {
	    throw new RemoteException("cannot add bean in container");
	}
    }

    /**
     * Remove beans
     */
    public void removeBeans(Vector descNames) throws RemoteException {
	Trace.out(trace, "Adm.removeBeans(Vector descNames) ");
	ecm.getContainer().removeBeans(descNames);
    }

    /**
     * List beans
     */
    public Vector listBeans() throws RemoteException {
	Trace.out(trace, "Adm.listBeans() ");
	return ecm.getContainer().listBeans();
    }
 
    /**
     * List JNDI context
     */
    public Vector listContext() {
	Trace.out(trace, "Adm.listContext() ");
	NamingEnumeration ne   = null;
	String            name = null;
	Vector            ret  = new Vector();
	String prop = null;
	try {
	    ne = ictx.list("");
	    for (Enumeration e = ne; e.hasMoreElements() ;) {
		name = ((NameClassPair) e.nextElement()).getName();
		Trace.outln(trace, "    name found in context = "+ name);
		ret.addElement(name);
	    }
	} catch (Exception ex) { 
	    Trace.errln("Cannot list JNDI context : " + prop);
	    Trace.errln(ex);
	    System.exit(1);
	}
	return ret;
    }

    /**
     * List Environment
     */
    public Properties listEnv() {
	Trace.out(trace, "Adm.listEnv() ");
	return Prop.svr.getFilesEnv();
    }

    /**
     * Stop the EJB Server
     */
    public void stopServer()   throws RemoteException {
	Trace.out(trace, "Adm.stopServer() ");
	ecm.getContainer().unbindBeans();
	DataBaseManager.getInstance().unbindDataSources();
	System.exit(0);
    }
 
    /**
     * To test if an Adm is reachable
     */
    public void ping() throws RemoteException {
    }
}
