/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2000 Bull S.A. All Rights Reserved.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container;

import java.util.Vector;

import com.lutris.classloader.MultiClassLoader;

import org.objectweb.jonas.common.ClassLoaderFactory;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.BeanDesc;
import org.objectweb.jonas.deployment.api.DeploymentDesc;
import org.objectweb.jonas.deployment.api.DeploymentDescException;
import org.objectweb.jonas.lib.BeanNaming;


public class EJBContainerManager {
    static private final int trace = Trace.DB_14;

    private static EJBContainerManager ejbcontainermanager = null;
    private static Container cont; // For the moment we have a unique Container
    private static MultiClassLoader loader;
    

    /*
     * We manage a unique instance
     *
     */

    private  EJBContainerManager() {

    }
    public static EJBContainerManager getInstance() {
	if (ejbcontainermanager == null){
	    ejbcontainermanager = new EJBContainerManager();
	    cont = new Container();
	    loader = new MultiClassLoader(null);
	}
	return ejbcontainermanager;
    }

    public void createEJBContainer(BeanDesc desc) throws Exception{
	Trace.outln(trace, "createEJBContainer()");
	cont.addBeans(desc);
    }

    /**
     * Containers creation for a set of Beans
     * in the current version of JOnAS there is a unique container
     * in which all beans are loaded. but it will change
     *
     * @param    names a list of files that may be xml files or ejb-jar files
     *
     */

    public void createEJBContainers(Vector names) throws Exception {
	Trace.outln(trace,"createEJBContainers "+names.toString()+") ");
	if (names == null) 
	    return;
	// Loops on ejb-jar
	for (int i = 0; i < names.size(); i++) {
	    String    tokenDesc = (String) names.elementAt(i);
	    DeploymentDesc ejbJarDD = null;
	    try {
		if (tokenDesc.endsWith(".jar")) {
		    // ejb-jar file
		    loader.addClassPath(tokenDesc);
		    ejbJarDD = DeploymentDesc.getInstance(loader);
		} else {
		    // xml file
		    ejbJarDD = DeploymentDesc.getInstance(tokenDesc,
							  BeanNaming.getJonasXmlName(tokenDesc));
		}
	    } catch (DeploymentDescException e) {
		Trace.errln("Cannot read the deployment descriptors '" + tokenDesc + "': " + e.getMessage());
		System.exit(1);
	    }
	    BeanDesc[] beansDD = ejbJarDD.getBeanDesc();
	    // Loops on beans in ejb-jar
	    for (int j=0; j<beansDD.length; j++) {
		BeanDesc  desc      = beansDD[j];
		createEJBContainer(desc);
	    }

	}
    }

    public Container getContainer() {
	return cont;
    }
	
	
}
