/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

/*
 * TODO
 *   - revoir getEnterpriseBeanClassName() et partout ou l'on manipule des noms de classes
 *     voir si on ne peut pas manipuler les classes elles memes !!!
 */


package org.objectweb.jonas.container;

import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.rmi.RemoteException;
import org.objectweb.jonas.common.RemoteObject;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import javax.ejb.EJBObject;
import javax.ejb.Handle;
import javax.ejb.HomeHandle;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.TransactionRolledbackException;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.deployment.api.BeanDesc;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.naming.NamingManager;


/**
 * This class implements javax.ejb.EJBHome interface.
 * This class must be extended by the EJBHome class generated for a given bean.
 * 
 * Contains all information related to the bean.
 */
public abstract class JBeanHome extends RemoteObject implements EJBHome  {

    private static final int trace = Trace.DB_16;

    /**
     * @serial
     */
    private BeanDesc ejbDescriptor = null; 

    /**
     * @serial
     */
    private JBeanMetaData ejbMetaData = null; 

    /**
     * @serial
     */
    private String beanName = null;

    /**
     * @serial
     */
    private Properties ejb10Env = null;

    /**
     * @serial
     */
    private HomeHandle handleEJBHome = null;

    /**
     * @serial
     */
    private boolean isSession = true;

    /**
     * @serial
     */
    protected boolean TxBeanManaged = false;

    /**
     * JNDI Context for the bean. 
     * @serial
     */
    protected CompNamingContext JNDICtx = null;

    /**
     * @serial
     */
    TransactionManager tm = Container.current;

    // ------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------

    /**
     * Constructor for the base class of the specific generated Home object.
     */
    public JBeanHome(BeanDesc desc, String beanName) throws RemoteException {

	Trace.outln(trace, "JBeanHome constructor for "+beanName);

	this.beanName   = beanName;
	ejbDescriptor   = desc;

	ejb10Env = desc.getEjb10Environment();

	register();
    }

    // ------------------------------------------------------------------
    // EJBHome implementation
    // ------------------------------------------------------------------

    /**
     * Obtains the EJBMetaData interface for the enterprise Bean. 
     * @return The enterprise Bean's EJBMetaData
     */
    public EJBMetaData getEJBMetaData() throws RemoteException {
	Trace.outln(trace, "JBeanHome.getEJBMetaData()");
	return ejbMetaData;
    }

    /**
     * Removes an EJB object identified by its handle. 
     */
    abstract public void remove(Handle handle) throws RemoteException, RemoveException;

    /**
     * Removes an EJB object identified by its primary key.
     */
    abstract public void remove(Object primaryKey) throws RemoteException, RemoveException;

    /**
     * Obtain a handle for the home object. The handle can be used at later time to re-obtain a
     * reference to the home object, possibly in a different Java Virtual Machine.
     * 
     * @return A handle for the home object.
     * 
     * @exception java.rmi.RemoteException - Thrown when the method failed due to a system-level
     * failure.
     */
    public HomeHandle getHomeHandle() throws java.rmi.RemoteException {

	Trace.outln(trace, "JBeanHome.getHomeHandle()");

	if (handleEJBHome == null) {
	    handleEJBHome = new HomeHandleImpl(getDeploymentDescriptor().getJndiName(), NamingManager.getEnv());
	}
	return handleEJBHome;
    }

    // ------------------------------------------------------------------
    // Other methods
    // ------------------------------------------------------------------

    /**
     * register this bean to JNDI (rebind)
     */
    public void register() {
	try {
	    NamingManager.registerBean(ejbDescriptor, this);
	} catch (Exception e) {
	    Trace.errln("Cannot register bean: " + e);
	    System.exit(1);
	}
    }

    /**
     * unregister this bean in JNDI (unbind)
     */
    public void unregister() {
	try {
	    NamingManager.unregisterBean(ejbDescriptor);
	} catch (Exception e) {
	    Trace.errln("Cannot unregister bean: " + e);
	    System.exit(1);
	}
    }

    public void setEJBMetaData(JBeanMetaData md) {
	Trace.outln (trace, "JBeanHome.setEJBMetaData() ");	
	ejbMetaData = md;
    }


    /**
     * return true if the bean manages itself the transactions
     */
    public boolean isTxBeanManaged() {
	return TxBeanManaged;
    }

    // DeploymentDescriptor
    public BeanDesc getDeploymentDescriptor() {
	Trace.outln (trace, "JBeanHome.getDeploymentDescriptor() ");
	return ejbDescriptor;
    }


    /**
     * Returns the EJB 1.0 style environment associated with the Bean
     */
    public Properties getEjb10Environment() {
	Trace.outln (trace, "JBeanHome.getEjb10Environment() for "+beanName);
	return ejb10Env;
    }

    /**
     * Gets the full name of the enterprise Bean's ClassName
     */
    public String getEnterpriseBeanClassName(){
	return ejbDescriptor.getEjbClass().getName();
    }

    /**
     * Set the JNDI Context for the bean
     */
    public void setJNDIContext(CompNamingContext ctx) {
	JNDICtx = ctx;
    }

    /**
     * Get the JNDI Context for the bean
     */
    public CompNamingContext getJNDIContext() {
	return JNDICtx;
    }

    // TODO: make a common code between this and business methods
    // See JBean.java - GenIC - ... (preinvoke + postinvoke)
    // Check that the tests covers all cases, including exceptions

    /**
     * Common preinvoke method for create and finder methods
     */
    abstract protected RequestCtx preinvoke(int txattr) throws RemoteException;
    
    /**
     * Common postinvoke method for create and finder methods
     */
    public void postinvoke(RequestCtx rctx) throws RemoteException {

	Trace.outln(trace, "JBeanHome.postinvoke");

	// Reset bean context for JNDI
	NamingManager.setComponentContext(rctx.getOldJNDICtx());

	// Commit or Rollback transaction started in preinvoke
	if (rctx.mustCommit()) {
	    if (rctx.getSysExc() != null) {
		// rollback tx
		try {
		    Trace.errln("System exc. => Rollback the transaction.");
		    tm.rollback();
		} catch (Exception e) {
		    Trace.errln("Exception during rollback:"+e);
		}
	    } else {
		// try to commit tx
		try {
		    Trace.outln(trace, "postInvoke: Committing transaction");
		    tm.commit();
		} catch (RollbackException e) {
		    // Probably a pb while writing data on database.
		    throw new RemoteException("Container cannot commit a transaction", e);
		} catch (Exception e) {
		    Trace.errln("postInvoke: " + e);
		    throw new RemoteException("Container exception", e);
		}
	    }
	}

	// We got a system Exception in create or finder method:
	// - Log system Exception
	// - discard instance (???)
	// - set client transaction rollback only
	// - throw remote exception
	if (rctx.getSysExc() != null) {
	    // Log system exception
	    Trace.errln("System Exception in create or finder method:"+rctx.getSysExc());

	    // If client transaction: set it rollback only and throw TransactionRolledBackException
	    Transaction currentTx = null;
	    try {
		currentTx = tm.getTransaction();
		if (currentTx != null) {
		    currentTx.setRollbackOnly();
		    throw new TransactionRolledbackException("System exception in business method");
		}
	    } catch (SystemException e) {
		Trace.errln("doPostInvoke: cannot set rollback only current tx:"+e);
	    }
	    return;
	}
	
	// Resume transaction context of the caller
	Transaction cltx = rctx.getClientTx();
	if (cltx != null) {
	    try {
		tm.resume(cltx);
	    } catch (SystemException e) {
		Trace.errln("postinvoke: cannot resume tx:\n"+e);
	    }
	}
    }

}
