/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jtm;

import java.rmi.RemoteException;
import javax.naming.NamingException;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.naming.NamingManager;

/**
 * Transaction Manager for an EJB Server.
 * this singleton class must exist in each jonas server.
 * all methods are static
 */
public class TransactionService {

    private static final int trace = Trace.DB_4;

    // TM factory may be local or remote.
    // The TM factory is used by the JTA implementation when transactions
    // are distributed among several JVM (EJBServer or applet client)
    private TransactionFactory tm = null;

    private static Current current = null;

    /**
     * Create the transaction manager.
     * arg jtmlocal true if we have to create a local jtm
     */
    public TransactionService(boolean jtmlocal) throws NamingException, RemoteException {

	// Gets the Distributed Transaction Manager (JTM)
	if (jtmlocal) {
	    // Creates a JTM locally in this server
	    Trace.outln(trace, "TransactionService: Working with a colocated Transaction Manager ");
	    
	    // TODO: Recovery of the JTM

	    // Creates the ControlFactory and register it in JNDI.
	    try {
		Trace.outln(trace, "Create and register TM factory");
		tm = new TransactionFactoryImpl();
		NamingManager.rebind("TMFactory", (Object) tm);
	    } catch (RemoteException e) {
		Trace.errln("TransactionService: Cannot create TransactionFactory:\n"+e);
		throw e;
	    } catch (NamingException e) {
		Trace.errln("TransactionService: Cannot rebind TM:\n"+e);
		throw e;
	    }
	} else {
	    // JTM is remote, finds it by JNDI
	    Trace.outln(trace, "TransactionService: Working with a remote Transaction Manager ");
	    try {
		tm = (TransactionFactory) NamingManager.lookup("TMFactory");
	    } catch (NamingException e) {
		Trace.errln("TransactionService: Cannot get TM factory:"+e);
		throw e;
	    }
	}
	
	// Create and init the unique Current object.
	// Current is the local TransactionManager. It must be present in every
	// jonas server and implements the JTA TransactionManager interface.
	current = new Current(tm);

	// Register a UserTransactionFactory in JNDI.
	// Only if we are inside the JTM
	if (jtmlocal) {
	    try {
		Trace.outln(trace, "Register UserTransactionFactory");
		NamingManager.rebind("java:comp/UserTransaction", current);
	    } catch (NamingException e) {
		Trace.errln("Cannot rebind UserTransaction:"+e);
		throw e;
	    }
	}

    }

    /**
     * Gets the Current object instance
     */
    public static Current getCurrent() {
	return current;
    }
}

